#/bin/bash
## Script per crear noves unitats de xarxa en SAMBA al servidor de centre Linkat edu 14.04 o superior.
## Desenvolupat per Pablo Vigo <pvigo#xtec.cat> <pablo.vigo#ticxcat.cat>

if [ -f /etc/Linkat/linkat-server/.entornvarlinkat ] && [ -f /etc/modalitat_linkat ]; then
	. /etc/Linkat/linkat-server/.entornvarlinkat
else
	zenity --error --title="No es poden afegir noves unitats" --text="El servidor de centre encara no ha sigut configurat\no l'usuari no és administrador del sistema."
	exit 2
fi

zenity --question --title="Nova unitat de xarxa SAMBA" --text="Aquest programa permet crear noves unitats\nde xarxa SAMBA al servidor de centre Linkat.\n\nVoleu continuar?"
if [ "$?" -gt 0 ] ; then
	exit 1
fi 

LOOP=0

while [ "$LOOP" -eq 0 ]
do

export TEMPDIR=$(mktemp -d)

res=$(zenity --forms --title="Unitats de xarxa SAMBA" --text="Crear nova unitat" --add-entry="Nom de la unitat" --add-entry="Grup LDAP")
if [ "$?" -gt 0 ]; then
	exit 1
fi

NOVAUNITAT=$(echo $res | awk 'BEGIN { FS="|"; } {print $1; }')
NOUGRUP=$(echo $res | awk 'BEGIN { FS="|"; } {print $2; }')

if [ "$NOVAUNITAT" == "" ] || [ "$NOUGRUP" == "" ] ; then
	zenity --error --text="S'ha d'introduir un nom d'unitat i un nom de grup."
	exit 2
fi

for i in "$NOVAUNITAT" "$NOUGRUP"
do
  echo "$i" | grep -E " "
  if [ "$?" == 0 ]; then
    zenity --error --text="No es poden fer servir espais."
    exit 2
  fi
done

res=$(egrep -R "^\[$NOVAUNITAT\]" /etc/samba/)
if [ "$?" == 0 ]; then
  zenity --info --title="Unitats de xarxa SAMBA" --text="Aquesta unitat de xarxa ja existeix al servidor.\n\nEl programa s'aturarà."
  exit 1
fi

## Creant el nou grup LDAP
ldapsearch -xLL | egrep cn: | sed "s/cn: //g" > $TEMPDIR/grupldap.list
res=$(egrep "^$NOUGRUP$" $TEMPDIR/grupldap.list)
        if [ "$?" == 0 ] ; then
          echo "Grup existent"
        else
          echo "Grup nou"
        GIDN=$(ldapsearch -xLL -H ldap://localhost | egrep gidNumber | awk -F: '{print $2}' | sort -n | tail -1)
        NOUGIDN=$(($GIDN + 1))
	cp -av /etc/Linkat/linkat-nova-unitat/nougrup.ldif.linkat /etc/Linkat/linkat-nova-unitat/nougrup.ldif
        	sed -i s/_NOUGRUP_/$NOUGRUP/g /etc/Linkat/linkat-nova-unitat/nougrup.ldif
           	sed -i s/_DOMINI_/$DOMINI/g /etc/Linkat/linkat-nova-unitat/nougrup.ldif
           	sed -i s/_NOUGIDN_/$NOUGIDN/g /etc/Linkat/linkat-nova-unitat/nougrup.ldif
	cp -av /etc/Linkat/linkat-nova-unitat/afegirnougrup.linkat $TEMPDIR/afegirnougrup
		sed -i s/_DOMINI_/$DOMINI/g $TEMPDIR/afegirnougrup
        	sed -i s/_PASSWDLDAP_/$PASS_LDAP/g $TEMPDIR/afegirnougrup
	chmod +x $TEMPDIR/afegirnougrup
	$TEMPDIR/afegirnougrup

        fi

## Creant unitat de xarxa
## Verificant si existeix als fitxers de configuració
res=$(egrep -R "^\[$NOVAUNITAT\]" /etc/samba/)
if [ "$?" == 0 ]; then
  zenity --info --title="Unitats de xarxa SAMBA" --text="Aquesta unitat de xarxa ja existeix al servidor.\n\nEl programa s'aturarà."
  exit 1
else

## Creant la nova unitat
  mkdir -m 6770 /srv/exports/$NOVAUNITAT
  chown root.$NOUGRUP /srv/exports/$NOVAUNITAT
  chmod -R 6770 /srv/exports/$NOVAUNITAT
  mkdir -p /srv/exports/Paperera/Paperera_Unitat_$NOVAUNITAT
  chmod -R 770 /srv/exports/Paperera/Paperera_Unitat_$NOVAUNITAT
  chown -R root:$NOUGRUP /srv/exports/Paperera/Paperera_Unitat_$NOVAUNITAT
  setfacl -m default:group::rwx /srv/exports/$NOVAUNITAT
  setfacl -R -m default:user:clamav:rwx -m user:clamav:rwx /srv/exports/$NOVAUNITAT
  setfacl -R -m default:user:clamav:rwx -m user:clamav:rwx /srv/exports/Paperera/Paperera_Unitat_$NOVAUNITAT
  setfacl -R -m default:user:lnadmin:rwx -m user:lnadmin:rwx /srv/exports/$NOVAUNITAT
  getfacl -p /srv/exports/$NOVAUNITAT | setfacl --set-file=- /srv/exports/Paperera/Paperera_Unitat_$NOVAUNITAT > /dev/null 2>&1
  chgrp -R $NOUGRUP /srv/exports/$NOVAUNITAT
fi

## Configurant el SAMBA
## Adaptant el nou fitxer novaunitat.conf
cp -av /etc/Linkat/linkat-nova-unitat/unitat.conf.linkat $TEMPDIR/$NOVAUNITAT.conf
sed -i s/_NOVAUNITAT_/$NOVAUNITAT/g $TEMPDIR/$NOVAUNITAT.conf
sed -i s/_NOUGRUP_/$NOUGRUP/g $TEMPDIR/$NOVAUNITAT.conf

## Copiant nou fitxer novaunitat.conf
if [ ! -d /etc/samba/smb.conf.d/ ] ; then
	mkdir -p /etc/samba/smb.conf.d/
fi
cp -av $TEMPDIR/$NOVAUNITAT.conf /etc/samba/smb.conf.d/

## Afegint nova linia a SMB.CONF
echo "include = /etc/samba/smb.conf.d/$NOVAUNITAT.conf" >> /etc/samba/smb.conf

## Eliminant el directori temporal
rm -rf $TEMPDIR
rm -rf /etc/Linkat/linkat-nova-unitat/nougrup.ldif

## Reniciant el servei SAMBA
service nmbd restart
service smbd restart

zenity --info --text="Nova unitat de SAMBA $NOVAUNITAT creada correctament."

zenity --question --title="Unitat de xarxa SAMBA" --text="Voleu afegir una altra unitat de xarxa SAMBA?"
if [ "$?" -gt 0 ]; then
        LOOP=1
fi

done

